﻿/*
 * Written by Gianni.Cavallotto@gmail.com
 * Date: 20110826
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Globalization;
using System.Linq;
using System.Threading;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.Misc;
using Infragistics.Win.UltraWinGrid;
using Infragistics.Win.UltraWinToolTip;
using Infragistics.Shared;

namespace UltraGridNavigatorSample
{
    /// <summary>
    /// UltraGridNavigator Class writen by Gianni.Cavallotto@gmail.com
    /// This class is intended to be used coupled with a Infragistics UltraGrid for Windows Form.
    /// </summary>
    public partial class UltraGridNavigator : UserControl
    {

        #region Fields

            private int currentRowIndex;
            private int currentRowsTotal;

            private UltraToolTipInfo onlyDigit;
            private UltraToolTipInfo outOfRange;
            private UltraToolTipInfo filterTip;
            private UltraToolTipInfo groupTip;

            private string lastSearchedText="";
            private int searchIndex;
            private List<UltraGridCell> searchedCells;
            private bool noMatchesFound;

            BackgroundWorker bgFinder = new BackgroundWorker();

            private bool caseInsensitive = true;
            private Dictionary<UltraGridCell, Color> originalActiveCellsBackColors = new Dictionary<UltraGridCell,Color>();
            private Dictionary<UltraGridCell, Color> originalCellsBackColors = new Dictionary<UltraGridCell,Color>();

            private bool backgroundSearching = false;

            long rowsTotal;

        #endregion


        public UltraGridNavigator()
        {
            InitializeComponent();
            InitializeComponentExtension();
            DeclaredManagedEvents();
        }

        private void DeclaredManagedEvents()
        {
            this.panelPosition.SizeChanged += new EventHandler(panelPosition_SizeChanged);
            this.panelSearch.SizeChanged += new EventHandler(panelSearch_SizeChanged);

            this.labelTotalRecords.SizeChanged += new EventHandler(labelTotalRecords_SizeChanged);

            this.buttonMoveFirst.Click += new EventHandler(buttonMoveFirst_Click);
            this.buttonMovePrevious.Click += new EventHandler(buttonMovePrevious_Click);
            this.buttonMoveLast.Click += new EventHandler(buttonMoveLast_Click);
            this.buttonMoveNext.Click += new EventHandler(buttonMoveNext_Click);

            this.textEditorActiveRecord.KeyPress += new KeyPressEventHandler(textEditorActiveRecord_KeyPress);
            this.textEditorActiveRecord.KeyUp += new KeyEventHandler(textEditorActiveRecord_KeyUp);
            this.textEditorActiveRecord.TextChanged += new EventHandler(textEditorActiveRecord_TextChanged);

            this.buttonSearch.Click += new EventHandler(buttonSearch_Click);
            this.textEditorSearch.KeyUp += new KeyEventHandler(textEditorSearch_KeyUp);
            this.buttonSearch.KeyUp += new KeyEventHandler(buttonSearch_KeyUp);

            this.bgFinder.DoWork += new DoWorkEventHandler(bgFinder_DoWork);
            this.bgFinder.RunWorkerCompleted += new RunWorkerCompletedEventHandler(bgFinder_RunWorkerCompleted);

            this.textEditorSearch.Leave += new EventHandler(textEditorSearch_Leave);
            this.buttonSearch.Leave += new EventHandler(buttonSearch_Leave);

            this.buttonFiltered.SizeChanged += new EventHandler(buttonFiltered_SizeChanged);
            this.buttonGrouped.SizeChanged += new EventHandler(buttonGrouped_SizeChanged);

            this.Disposed += new EventHandler(UltraGridNavigator_Disposed);
        }


        #region Declared Events Implementations

            void buttonGrouped_SizeChanged(object sender, EventArgs e)
            {
                this.buttonGrouped.Height = this.panelPosition.Height-2;
                this.buttonGrouped.Top = 1;
                this.RePositioning();
            }

            void buttonFiltered_SizeChanged(object sender, EventArgs e)
            {
                this.buttonFiltered.Height = this.panelPosition.Height - 2;
                this.buttonFiltered.Top = 1;
                this.RePositioning();
            }

            void buttonSearch_Leave(object sender, EventArgs e)
            {
                if (!this.textEditorSearch.Focused && !this.buttonSearch.Focused) {
                    this.RestoreCellsColor();
                }
            }

            void textEditorSearch_Leave(object sender, EventArgs e)
            {
                if (!this.textEditorSearch.Focused && !this.buttonSearch.Focused) {
                    this.RestoreCellsColor();
                }
            }

            void buttonSearch_KeyUp(object sender, KeyEventArgs e)
            {
                if (e.KeyCode == Keys.F3) {
                    this.buttonSearch_Click(this, new System.EventArgs());
                } else if (e.KeyCode == Keys.F && e.Control) {
                    this.textEditorSearch.Focus();
                    this.textEditorSearch.SelectAll();
                }
            }

            void textEditorSearch_KeyUp(object sender, KeyEventArgs e)
            {
                if (e.KeyCode == Keys.F3 || e.KeyCode==Keys.Enter || e.KeyCode==Keys.Return) {
                    this.buttonSearch_Click(this, new System.EventArgs());
                } else if (e.KeyCode == Keys.F && e.Control) {
                    this.textEditorSearch.Focus();
                    this.textEditorSearch.SelectAll();
                }
            }

            void bgFinder_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
            {
                if (this.noMatchesFound && !this.bgFinder.CancellationPending && !this.backgroundSearching) {
                    string msg = String.Format(CultureInfo.CurrentUICulture, UltraGridNavigatorResources.NoMachesFound, this.lastSearchedText);
                    MessageBox.Show(msg, UltraGridNavigatorResources.Search_CAPTION, MessageBoxButtons.OK, MessageBoxIcon.Information, MessageBoxDefaultButton.Button1, 0);
                }
            }

            void bgFinder_DoWork(object sender, DoWorkEventArgs e)
            {
                string searchedText = e.Argument.ToString();
                searchedText = this.caseInsensitive ? searchedText.ToLower(CultureInfo.CurrentUICulture) : searchedText;
                // Loop through all rows looking for searched Text
                foreach (UltraGridRow row in this.SourceGrid.DisplayLayout.Rows.GetFilteredInNonGroupByRows()) {
                    if (this.bgFinder.CancellationPending) {
                        break;
                    }
                    if (row.IsDataRow) {
                        foreach (UltraGridCell cell in row.Cells) {
                            string cellText;
                            cellText = this.caseInsensitive ? cell.Text.ToLower(CultureInfo.CurrentUICulture) : cell.Text;
                            if (cellText.Contains(searchedText)) {
                                this.searchedCells.Add(cell);
                                MarkSearchedCell(cell);
                            }
                        }
                    }
                }
                this.noMatchesFound = this.searchedCells.Count == 0;
            }

            void buttonSearch_Click(object sender, EventArgs e)
            {
                FindByText(this.textEditorSearch.Text, false);
            }

            private void FindByText(string textToFind, bool matchCase)
            {
                if (!String.IsNullOrEmpty(textToFind)) {
                    if (textToFind != this.lastSearchedText || this.searchedCells == null) {
                        this.noMatchesFound = false;
                        this.lastSearchedText = textToFind;

                        SearchContents(textToFind, matchCase);

                        if (!this.noMatchesFound) {
                            this.SourceGrid.ActiveCell = this.searchedCells[0];
                            this.BringActiveCellIntoView();
                            this.searchIndex = 0;
                        }
                        
                    } else {

                        if (this.searchIndex == -2) { //arrivo da una precedente ricerca dello stesso testo
                            foreach (UltraGridCell cell in this.searchedCells) {
                                this.MarkSearchedCell(cell);
                            }
                            this.searchIndex = -1;
                        }

                        this.searchIndex++;
                        if (this.searchIndex > this.searchedCells.Count - 1) {
                            this.searchIndex = 0;
                        }

                        if (this.searchedCells.Count > this.searchIndex) {
                            this.SourceGrid.ActiveCell = this.searchedCells[this.searchIndex];
                            this.BringActiveCellIntoView();
                        }
                    }

                    if (this.searchedCells.Count > 0) {
                        if (this.originalActiveCellsBackColors.Count > 0) {
                            int prevIndex = this.searchIndex - 1;
                            if (prevIndex == -1)
                                prevIndex = this.searchedCells.Count - 1;
                            if (this.originalActiveCellsBackColors.ContainsKey(this.searchedCells[prevIndex])) {
                                this.searchedCells[prevIndex].ActiveAppearance.BackColor = this.originalActiveCellsBackColors[this.searchedCells[prevIndex]];
                            }
                        }

                        if (this.originalActiveCellsBackColors.ContainsKey(this.SourceGrid.ActiveCell)) {
                            this.originalActiveCellsBackColors.Remove(this.SourceGrid.ActiveCell);
                        }
                        this.originalActiveCellsBackColors.Add(this.SourceGrid.ActiveCell, this.SourceGrid.ActiveCell.ActiveAppearance.BackColor);

                        this.SourceGrid.ActiveCell.ActiveAppearance.BackColor = this.SearchResultColorHighlight;
                    }
                }
            }

            private void SearchContents(string textToSearch, bool matchCase)
            {
                using (new WaitCursor()) {

                    this.RestoreCellsColor();
                    this.searchedCells = new List<UltraGridCell>();
                    //find rows in a parallel thread
                    if (this.bgFinder.IsBusy) {
                        //If still working stops previous search
                        this.bgFinder.CancelAsync();
                    }
                    while (this.bgFinder.IsBusy) {
                        Application.DoEvents();
                    }

                    this.caseInsensitive = !matchCase;

                    this.bgFinder.RunWorkerAsync(textToSearch);
                    while (!this.noMatchesFound && this.searchedCells.Count == 0) {
                        Thread.Sleep(100);
                    }
                }
            }

            void textEditorActiveRecord_KeyUp(object sender, KeyEventArgs e)
            {
                if (e.KeyData == Keys.Enter || e.KeyData == Keys.Return) {
                    this.currentRowIndex = Convert.ToInt32(this.textEditorActiveRecord.Text.TrimEnd(), CultureInfo.CurrentUICulture);
                    if (this.filtered && !this.grouped) {
                        this.SourceGrid.ActiveRow = this.SourceGrid.Rows.GetFilteredInNonGroupByRows()[this.currentRowIndex - 1];
                    } else {
                        int index = this.currentRowIndex - 1;
                        if (index < 0)
                            index = 0;
                        if (index == 0 && !this.SourceGrid.Rows.FilterRow.Hidden) {
                            this.SourceGrid.ActiveRow = this.SourceGrid.Rows.FilterRow;
                        } else {
                            this.SourceGrid.ActiveRow = this.GetSameGroupRows(this.SourceGrid.Rows.ToList(), this.SourceGrid.ActiveRow)[index];
                        }
                    }
                }
            }

            void textEditorActiveRecord_TextChanged(object sender, EventArgs e)
            {
                if (!String.IsNullOrEmpty(this.textEditorActiveRecord.Text.Trim()) ) {
                    int currentRow = Convert.ToInt32(this.textEditorActiveRecord.Text.TrimEnd(), CultureInfo.CurrentUICulture);
                    if (currentRow > this.currentRowsTotal) {
                        this.ultraToolTipManager1.SetUltraToolTip(this, this.outOfRange);
                        this.ultraToolTipManager1.ShowToolTip(this, this.textEditorActiveRecord.Parent.PointToScreen(new Point(this.textEditorActiveRecord.Left + 5, this.textEditorActiveRecord.Top + this.textEditorActiveRecord.Height - 1)));
                        if (this.currentRowIndex < this.currentRowsTotal)
                        {
                            this.textEditorActiveRecord.Text = this.currentRowIndex.ToString(CultureInfo.CurrentUICulture);
                        }
                        else
                        {
                            this.textEditorActiveRecord.Text = " ";
                        }
                    }
                    if (this.textEditorActiveRecord.Text[this.textEditorActiveRecord.Text.Length - 1] != ' ') {
                        this.textEditorActiveRecord.Text += " ";
                    }
                }
            }

            void textEditorActiveRecord_KeyPress(object sender, KeyPressEventArgs e)
            {
                if (!Char.IsDigit(e.KeyChar) && !Char.IsControl(e.KeyChar)) {
                    this.ultraToolTipManager1.SetUltraToolTip(this, this.onlyDigit);
                    this.ultraToolTipManager1.ShowToolTip(this, this.textEditorActiveRecord.Parent.PointToScreen(new Point(this.textEditorActiveRecord.Left + 5, this.textEditorActiveRecord.Top + this.textEditorActiveRecord.Height - 1)));
                    e.Handled = true;
                }
            }
  
            void panelPosition_SizeChanged(object sender, EventArgs e)
            {
                this.textEditorActiveRecord.Top = (this.panelPosition.Height - this.textEditorActiveRecord.Height) / 2;
            }

            void panelSearch_SizeChanged(object sender, EventArgs e)
            {
                this.textEditorSearch.Top = (this.panelSearch.Height - this.textEditorSearch.Height) / 2;
            }

            void labelTotalRecords_SizeChanged(object sender, EventArgs e)
            {
                RePositioning();
            }

            void buttonMoveNext_Click(object sender, EventArgs e)
            {
                if (this.SourceGrid != null) {
                    if (this.currentRowIndex < this.currentRowsTotal) {
                        if (this.SourceGrid.ActiveRow == null) {
                            UltraGridRow[] rows = this.SourceGrid.DisplayLayout.Rows.GetFilteredInNonGroupByRows();
                            if (rows.Count() > 0) {
                                this.SourceGrid.ActiveRow = rows[0];
                            }
                        } else {
                            this.SourceGrid.PerformAction(UltraGridAction.BelowRow);
                        }
                    }
                }
            }

            void buttonMoveLast_Click(object sender, EventArgs e)
            {
                if (this.SourceGrid != null) {
                    if (this.currentRowIndex < this.currentRowsTotal) {
                        if (this.SourceGrid.ActiveRow == null) {
                            UltraGridRow[] rows = this.SourceGrid.DisplayLayout.Rows.GetFilteredInNonGroupByRows();
                            if (rows.Count() > 0) {
                                this.SourceGrid.ActiveRow = rows[rows.Count() - 1];
                            }
                        } else {
                            this.SourceGrid.PerformAction(UltraGridAction.LastRowInBand);
                        }
                    }
                }
            }

            void buttonMovePrevious_Click(object sender, EventArgs e)
            {
                if (this.SourceGrid != null) {
                    if (this.currentRowIndex > 1) {
                        if (this.SourceGrid.ActiveRow == null) {
                            UltraGridRow[] rows = this.SourceGrid.DisplayLayout.Rows.GetFilteredInNonGroupByRows();
                            if (rows.Count() > 0) {
                                this.SourceGrid.ActiveRow = rows[0];
                            }
                        } else {
                            this.SourceGrid.PerformAction(UltraGridAction.AboveRow);
                        }
                    }
                }
            }

            void buttonMoveFirst_Click(object sender, EventArgs e)
            {
                if (this.SourceGrid != null) {
                    if (this.currentRowIndex > 1) {
                        if (this.SourceGrid.ActiveRow == null) {
                            UltraGridRow[] rows = this.SourceGrid.DisplayLayout.Rows.GetFilteredInNonGroupByRows();
                            if (rows.Count() > 0) {
                                this.SourceGrid.ActiveRow = rows[0];
                            }
                        } else {
                            this.SourceGrid.PerformAction(UltraGridAction.FirstRowInBand);
                            if (this.SourceGrid.ActiveRow == this.SourceGrid.Rows.FilterRow) {
                                this.SourceGrid.PerformAction(UltraGridAction.BelowRow);
                            }
                        }
                    }
                }
            }

            void UltraGridNavigator_Disposed(object sender, EventArgs e)
            {
                this.UnSubscribeSourceGridEvents();

                this.panelPosition.SizeChanged -= panelPosition_SizeChanged;
                this.panelSearch.SizeChanged -= panelSearch_SizeChanged;

                this.labelTotalRecords.SizeChanged -= labelTotalRecords_SizeChanged;

                this.buttonMoveFirst.Click -= buttonMoveFirst_Click;
                this.buttonMovePrevious.Click -= buttonMovePrevious_Click;
                this.buttonMoveLast.Click -= buttonMoveLast_Click;
                this.buttonMoveNext.Click -= buttonMoveNext_Click;

                this.textEditorActiveRecord.KeyPress -= textEditorActiveRecord_KeyPress;
                this.textEditorActiveRecord.KeyUp -= textEditorActiveRecord_KeyUp;
                this.textEditorActiveRecord.TextChanged -= textEditorActiveRecord_TextChanged;

                this.buttonSearch.Click -= buttonSearch_Click;
                this.textEditorSearch.KeyUp -= textEditorSearch_KeyUp;
                this.buttonSearch.KeyUp -= buttonSearch_KeyUp;

                this.bgFinder.DoWork -= bgFinder_DoWork;
                this.bgFinder.RunWorkerCompleted -= bgFinder_RunWorkerCompleted;

                this.textEditorSearch.Leave -= textEditorSearch_Leave;
                this.buttonSearch.Leave -= buttonSearch_Leave;

                this.buttonFiltered.SizeChanged -= buttonFiltered_SizeChanged;
                this.buttonGrouped.SizeChanged -= buttonGrouped_SizeChanged;

                this.Disposed -= UltraGridNavigator_Disposed;

                this.ultraToolTipManager1.ContainingControl = null;
            }

            void sourceGrid_AfterRowActivate(object sender, EventArgs e)
            {
                this.UpdateRowPosition(this.SourceGrid);
            }

            void sourceGrid_PropertyChanged(object sender, Infragistics.Win.PropertyChangedEventArgs e)
            {
                PropChangeInfo columnFilterChangeInfo = e.ChangeInfo.FindPropId(Infragistics.Win.UltraWinGrid.PropertyIds.ColumnFilters);
                if (columnFilterChangeInfo != null) {
                    this.UpdateRowPosition(this.SourceGrid);
                    this.filtered = this.GridIsFiltered();
                }
            }

            void sourceGrid_AfterSortChange(object sender, BandEventArgs e)
            {
                this.grouped = this.GridIsGrouped();
            }

            void sourceGrid_Paint(object sender, PaintEventArgs e)
            {
                if (this.sourceGrid.Rows.Count != this.rowsTotal) {
                    this.UpdateRowPosition(this.sourceGrid);
                    this.rowsTotal = this.sourceGrid.Rows.Count;
                }
            }


        #endregion


        #region Properties

            private UltraGrid sourceGrid;
            public UltraGrid SourceGrid
            {
                get
                {
                    return this.sourceGrid;
                }
                set
                {
                    if (this.sourceGrid != null) {
                        this.UnSubscribeSourceGridEvents();
                    }
                    this.sourceGrid = value;
                    if (this.sourceGrid != null) {
                        this.sourceGrid.AfterRowActivate += new EventHandler(sourceGrid_AfterRowActivate);
                        this.sourceGrid.PropertyChanged += new Infragistics.Win.PropertyChangedEventHandler(sourceGrid_PropertyChanged);
                        this.sourceGrid.AfterSortChange += new BandEventHandler(sourceGrid_AfterSortChange);
                        this.sourceGrid.Paint += new PaintEventHandler(sourceGrid_Paint);

                        if (this.sourceGrid.DisplayLayout.Override.ActiveRowAppearance.BackColor != Color.Lime) {
                            this.SearchResultColorHighlight = Color.Lime;
                            this.SearchResultColor = Color.FromArgb(0xD9, 0xFE, 0xD9);
                        } else {
                            this.SearchResultColorHighlight = Color.Yellow;
                            this.SearchResultColor = Color.FromArgb(0xFF, 0xFF, 0xE2);
                        }

                        this.filtered = this.GridIsFiltered();
                        this.grouped = this.GridIsGrouped();

                        if (this.SourceGrid.Rows.FilteredInNonGroupByRowCount == 0)
                            this.UpdateRowPosition(this.SourceGrid);
                    }
                }
            }

            
            public Color SearchResultColor
            {
                get;
                set;
            }
            public Color SearchResultColorHighlight
            {
                get;
                set;
            }

            private bool _filtered;
            private bool filtered
            {
                get
                {
                    return this._filtered;
                }
                set
                {
                    this._filtered = value;
                    if (this._filtered) {
                        this.filterTip.ToolTipText = UltraGridNavigatorResources.filterOn_TIP;
                        this.filterTip.ToolTipTitle = UltraGridNavigatorResources.filterOn_TIP_TITLE;
                        this.buttonFiltered.Appearance.FontData.Bold = DefaultableBoolean.True;
                        this.buttonFiltered.Text = UltraGridNavigatorResources.buttonFilteredON;
                        this.buttonFiltered.Appearance.Image = UltraGridNavigatorResources.Filter.ToBitmap();
                        this.buttonFiltered.Enabled = true;
                    } else {
                        this.filterTip.ToolTipText = UltraGridNavigatorResources.filterOff_TIP;
                        this.filterTip.ToolTipTitle = UltraGridNavigatorResources.filterOff_TIP_TITLE;
                        this.buttonFiltered.Appearance.FontData.Bold = DefaultableBoolean.False;
                        this.buttonFiltered.Text = UltraGridNavigatorResources.buttonFilteredOFF;
                        this.buttonFiltered.Appearance.Image = UltraGridNavigatorResources.Filter.ToBitmap();
                        this.buttonFiltered.Enabled = false;
                    }
                }
                    
            }

            private bool _grouped;
            private bool grouped
            {
                get
                {
                    return this._grouped;
                }
                set
                {
                    this._grouped = value;
                    if (this._grouped) {
                        this.groupTip.ToolTipText = UltraGridNavigatorResources.groupOn_TIP;
                        this.groupTip.ToolTipTitle = UltraGridNavigatorResources.groupOn_TIP_TITLE;
                        this.buttonGrouped.Appearance.FontData.Bold = DefaultableBoolean.True;
                        this.buttonGrouped.Text = UltraGridNavigatorResources.buttonGroupedON;
                        this.buttonGrouped.Appearance.Image = UltraGridNavigatorResources.Grouped.ToBitmap();
                        this.buttonGrouped.Enabled = true;
                    } else {
                        this.groupTip.ToolTipText = UltraGridNavigatorResources.groupOff_TIP;
                        this.groupTip.ToolTipTitle = UltraGridNavigatorResources.groupOff_TIP_TITLE;
                        this.buttonGrouped.Appearance.FontData.Bold = DefaultableBoolean.False;
                        this.buttonGrouped.Text = UltraGridNavigatorResources.buttonGroupedOFF;
                        this.buttonGrouped.Appearance.Image = UltraGridNavigatorResources.Grouped.ToBitmap();
                        this.buttonGrouped.Enabled = false;
                    }
                }

            }
            
        #endregion


        #region Overridden Methods

            protected override void OnLoad(EventArgs e)
            {
                base.OnLoad(e);

                UltraPanel mokePanel = new UltraPanel();
                mokePanel.UseAppStyling = true;

                AppearanceData appearance = new Infragistics.Win.AppearanceData();
                AppearancePropFlags flags = AppearancePropFlags.AllRenderAndCursor;
                mokePanel.ResolveAppearance(ref appearance, ref flags);

                this.ApplyAppearanceData(this.panelBack.Appearance, appearance); 
                this.panelBack.BorderStyle = UIElementBorderStyle.None;
                this.ApplyAppearanceData(this.panelPosition.Appearance, appearance); 
                this.panelPosition.BorderStyle = UIElementBorderStyle.None;
                this.ApplyAppearanceData(this.panelSearch.Appearance, appearance); 
                this.panelSearch.BorderStyle = UIElementBorderStyle.None;
            }

        #endregion


        #region Private Methods

            private void ApplyAppearanceData(AppearanceBase appearance, AppearanceData appearanceData)
            {
                appearance.BackColor = appearanceData.BackColor;
                appearance.BackColor2 = appearanceData.BackColor2;
                appearance.BackColorAlpha = appearanceData.BackColorAlpha;
                appearance.BackGradientAlignment = appearanceData.BackGradientAlignment;
                appearance.BackGradientStyle = appearanceData.BackGradientStyle;
                appearance.BackHatchStyle = appearanceData.BackHatchStyle;
            }

            private void UnSubscribeSourceGridEvents()
            {
                this.sourceGrid.AfterRowActivate -= this.sourceGrid_AfterRowActivate;
                //this.sourceGrid.AfterRowFilterChanged -= this.sourceGrid_AfterRowFilterChanged;
                this.sourceGrid.PropertyChanged -= this.sourceGrid_PropertyChanged;
                this.sourceGrid.AfterSortChange -= this.sourceGrid_AfterSortChange;
            }

            private void InitializeComponentExtension()
            {
                this.labelTotalRecords.Text = "0";

                NoFocusRectDrawFilter filter = new NoFocusRectDrawFilter();
                this.ultraToolTipManager1.SetUltraToolTip(this.buttonMoveFirst, new UltraToolTipInfo(UltraGridNavigatorResources.buttonMoveFirst_TIP, Infragistics.Win.ToolTipImage.None, "", Infragistics.Win.DefaultableBoolean.True));
                this.buttonMoveFirst.Appearance.ImageBackground = UltraGridNavigatorResources.ArrowPrevious.ToBitmap();
                this.buttonMoveFirst.Cursor = Cursors.Hand;
                this.buttonMoveFirst.DrawFilter = filter;

                this.ultraToolTipManager1.SetUltraToolTip(this.buttonMovePrevious, new UltraToolTipInfo(UltraGridNavigatorResources.buttonMovePrevious_TIP, Infragistics.Win.ToolTipImage.None, "", Infragistics.Win.DefaultableBoolean.True));
                this.buttonMovePrevious.Appearance.ImageBackground = UltraGridNavigatorResources.ArrowBack.ToBitmap();
                this.buttonMovePrevious.Cursor = Cursors.Hand;
                this.buttonMovePrevious.DrawFilter = filter;

                this.ultraToolTipManager1.SetUltraToolTip(this.buttonMoveLast, new UltraToolTipInfo(UltraGridNavigatorResources.buttonMoveLast_TIP, Infragistics.Win.ToolTipImage.None, "", Infragistics.Win.DefaultableBoolean.True));
                this.buttonMoveLast.Appearance.ImageBackground = UltraGridNavigatorResources.ArrowNext.ToBitmap();
                this.buttonMoveLast.Cursor = Cursors.Hand;
                this.buttonMoveLast.DrawFilter = filter;

                this.ultraToolTipManager1.SetUltraToolTip(this.buttonMoveNext, new UltraToolTipInfo(UltraGridNavigatorResources.buttonMoveNext_TIP, Infragistics.Win.ToolTipImage.None, "", Infragistics.Win.DefaultableBoolean.True));
                this.buttonMoveNext.Appearance.ImageBackground = UltraGridNavigatorResources.ArrowForward.ToBitmap();
                this.buttonMoveNext.Cursor = Cursors.Hand;
                this.buttonMoveNext.DrawFilter = filter;

                this.ultraToolTipManager1.SetUltraToolTip(this.buttonSearch, new UltraToolTipInfo(UltraGridNavigatorResources.buttonSearch_TIP, Infragistics.Win.ToolTipImage.None, "", Infragistics.Win.DefaultableBoolean.True));
                this.buttonSearch.Appearance.ImageBackground = UltraGridNavigatorResources.Search.ToBitmap();
                this.buttonSearch.Cursor = Cursors.Hand;
                this.buttonSearch.DrawFilter = filter;

                this.onlyDigit = new UltraToolTipInfo(UltraGridNavigatorResources.onlyDigits_TIP, Infragistics.Win.ToolTipImage.Warning, UltraGridNavigatorResources.onlyDigits_TIP_TITLE, Infragistics.Win.DefaultableBoolean.True);
                this.outOfRange = new UltraToolTipInfo(UltraGridNavigatorResources.outOfRange_TIP, Infragistics.Win.ToolTipImage.Warning, UltraGridNavigatorResources.outOfRange_TIP_TITLE, Infragistics.Win.DefaultableBoolean.True);

                this.filterTip = new UltraToolTipInfo("", ToolTipImage.None, "", DefaultableBoolean.True);
                this.ultraToolTipManager1.SetUltraToolTip(this.buttonFiltered, this.filterTip);
                this.groupTip = new UltraToolTipInfo("", ToolTipImage.None, "", DefaultableBoolean.True);
                this.ultraToolTipManager1.SetUltraToolTip(this.buttonGrouped, this.groupTip);

                this.bgFinder.WorkerSupportsCancellation = true;

                this.buttonFiltered.DrawFilter = filter;
                this.buttonGrouped.DrawFilter = filter;
            }

            private bool GridIsFiltered(){
                bool retVal = false;
                foreach (UltraGridBand band in this.SourceGrid.DisplayLayout.Bands) {
                    foreach (UltraGridColumn column in band.Columns) {
                        if (band.ColumnFilters[column].FilterConditions.Count > 0) {
                            retVal = true;
                            break;
                        }
                    }
                    if (retVal)
                        break;
                }
                return retVal;
            }

            private bool GridIsGrouped()
            {
                bool retVal = false;
                foreach (UltraGridBand band in this.SourceGrid.DisplayLayout.Bands) {
                    foreach (UltraGridColumn column in band.SortedColumns) {
                        if (column.IsGroupByColumn){
                            retVal = true;
                            break;
                        }
                    }
                    if (retVal)
                        break;
                }
                return retVal;
            }

            private void BringActiveCellIntoView()
            {
                RowScrollRegion rsr = this.SourceGrid.ActiveRowScrollRegion;
                ColScrollRegion csr = this.SourceGrid.ActiveColScrollRegion;

                rsr.ScrollCellIntoView(this.SourceGrid.ActiveCell, csr);
            }

            private void RestoreCellsColor()
            {
                foreach (UltraGridCell cell in this.originalActiveCellsBackColors.Keys) {       
                        cell.ActiveAppearance.BackColor = originalActiveCellsBackColors[cell];
                }
                if (this.searchedCells != null) {
                    foreach (UltraGridCell cell in this.searchedCells) {
                        if (this.originalCellsBackColors.ContainsKey(cell))
                            cell.Appearance.BackColor = this.originalCellsBackColors[cell];
                    }
                }
                Application.DoEvents();
                
                this.originalActiveCellsBackColors.Clear();
                this.originalCellsBackColors.Clear();
                if (this.searchedCells != null && this.searchedCells.Count>0)
                    this.searchIndex = -2;
            }

            private void UpdateRowPosition(UltraGrid ultraGrid)
            {
                if (ultraGrid.ActiveRow != null)
                {

                    this.currentRowIndex = ultraGrid.ActiveRow.VisibleIndex + 1;

                    //Load grouped columns
                    List<UltraGridColumn> groupedColumns = new List<UltraGridColumn>();
                    foreach (UltraGridColumn uGC in ultraGrid.DisplayLayout.Bands[0].SortedColumns)
                    {
                        if (uGC.IsGroupByColumn)
                        {
                            groupedColumns.Add(uGC);
                        }
                    }
                    if (groupedColumns.Count > 0)
                    {
                        // The grid is grouped by some columns
                        UltraGridGroupByRow groupRow = ultraGrid.ActiveRow as UltraGridGroupByRow;
                        if (groupRow != null)
                        {
                            // ActiveRow is the groupHeader
                            this.currentRowIndex = 0;
                        }
                        if (ultraGrid.ActiveRow == ultraGrid.Rows.FilterRow)
                        {
                            // ActiveRow is the FilterRow
                            this.currentRowIndex = 0;
                        }
                        this.currentRowsTotal = this.GetSameGroupRows(this.SourceGrid.Rows.ToList(), this.SourceGrid.ActiveRow).Count;
                    }
                    else
                    {
                        // The grid is not grouped
                        if (this.SourceGrid.Rows.FilterRow != null && !this.SourceGrid.Rows.FilterRow.Hidden)
                        {
                            this.currentRowIndex--;
                        }
                        this.currentRowsTotal = ultraGrid.DisplayLayout.Rows.FilteredInNonGroupByRowCount;
                    }

                    this.textEditorActiveRecord.Text = this.currentRowIndex.ToString(CultureInfo.CurrentUICulture);
                    this.labelTotalRecords.Text = this.currentRowsTotal.ToString(CultureInfo.CurrentUICulture);
                }
                else
                {
                    this.textEditorActiveRecord.Text = "0";
                    this.labelTotalRecords.Text = "0";
                }
            }

            private List<UltraGridRow> GetSameGroupRows(List<UltraGridRow> fullRowsSet, UltraGridRow ultraGridRow)
            {
                bool rowIsGroupHeader = ultraGridRow is UltraGridGroupByRow;

                List<UltraGridRow> rows = new List<UltraGridRow>();
                foreach (UltraGridRow row in fullRowsSet) {
                    if (row.IsGroupByRow) {
                        UltraGridGroupByRow gRow = row as UltraGridGroupByRow;
                        if (gRow != null) {
                            if (!rowIsGroupHeader && !gRow.Value.Equals(ultraGridRow.Cells[gRow.Column.Key].Value) ){
                                continue;
                            } else if (!rowIsGroupHeader) {
                                rows = GetSameGroupRows(gRow.Rows.ToList(), ultraGridRow);
                            }
                        }
                    } else {
                        if (!row.IsFilteredOut) {
                            rows.Add(row);
                        }
                    }
                }
                return rows;
            }

            private void MarkSearchedCell(UltraGridCell cell)
            {
                if (this.originalCellsBackColors.ContainsKey(cell)) {
                    this.originalCellsBackColors.Remove(cell);
                }
                this.originalCellsBackColors.Add(cell, cell.Appearance.BackColor);
                cell.Appearance.BackColor = this.SearchResultColor;
            }

            private void RePositioning()
            {
                this.labelTotalRecords.Top = ((this.panelPosition.Height - this.labelTotalRecords.Height) / 2) + ((this.panelPosition.Height - this.labelTotalRecords.Height) % 2);
                this.buttonMoveNext.Left = this.labelTotalRecords.Left + this.labelTotalRecords.Width + 4;
                this.buttonMoveLast.Left = this.buttonMoveNext.Left + this.buttonMoveNext.Width;
                this.labelSeparator1.Left = this.buttonMoveLast.Left + this.buttonMoveLast.Width + 10;
                this.buttonFiltered.Left = this.labelSeparator1.Left + this.labelSeparator1.Width + 4;
                this.labelSeparator2.Left = this.buttonFiltered.Left + this.buttonFiltered.Width + 4;
                this.buttonGrouped.Left = this.labelSeparator2.Left + this.labelSeparator2.Width + 4;
                this.labelSeparator3.Left = this.buttonGrouped.Left + this.buttonGrouped.Width + 4;
                this.panelPosition.Width = this.labelSeparator3.Left + this.labelSeparator3.Width + 4;
                this.panelSearch.Left = this.panelPosition.Width + 1;
            }

        #endregion


        #region Public Methods

            public IEnumerable<UltraGridCell> Search(string textToSearch,bool matchCase)
            {
                this.backgroundSearching = true;

                this.SearchContents(textToSearch, matchCase);

                while (this.bgFinder.IsBusy) {
                    Application.DoEvents();
                }

                this.backgroundSearching = false;

                return this.searchedCells;
            }

        #endregion


    }
}
